/****************************************************
	MARC Template Library 

  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: mark_basedow@hotmail.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/


#ifndef _DYNAMIC_MARCRECORD_H
#define _DYNAMIC_MARCRECORD_H

#include <string>
#include "marcutil.h"
#include "marcfield.h"
#include "marcleader.h"
#include "marcdir.h"
#include "marcrecord.h"
#include "marcsort.h"
#include <vector>


using namespace std;



template<class TLeader, class TDir>
class DynamicMARCRecord : public MARCRecord<TLeader, TDir>
{
public:
	DynamicMARCRecord();
	virtual ~DynamicMARCRecord();
	DynamicMARCRecord(const DynamicMARCRecord<TLeader, TDir> & mr);
	
	bool Parse(unsigned char * szBuf, int iBufLength);

	void MoveFirst();
	void MoveNext();
	bool IsEnd();

	bool FindFirst(char * szTag);
	bool FindNext(char * szTag);

	MARCTag Tag();
	MARCField & Field();

	string GetTitleString(char * sFzTag);

	DynamicMARCRecord<TLeader, TDir> & operator=(const DynamicMARCRecord<TLeader, TDir> & rec);
	bool operator<(const DynamicMARCRecord<TLeader, TDir> & rec) const;

protected:

	vector<MARCField>::iterator pos;
	vector<MARCField> fields;

};

template<class TLeader, class TDir>
DynamicMARCRecord<TLeader, TDir>::DynamicMARCRecord()
{
	
}

template<class TLeader, class TDir>
DynamicMARCRecord<TLeader, TDir>::~DynamicMARCRecord()
{

}

template<class TLeader, class TDir>
DynamicMARCRecord<TLeader, TDir>::DynamicMARCRecord(const DynamicMARCRecord<TLeader, TDir> & rec)
{
	leader = rec.leader;
	bValid = rec.bValid;
	iLength = rec.iLength;

	pos = rec.pos;
	fields = rec.fields;

}


template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::Parse(unsigned char * szBuf, int iLen)
{
	fields.clear();

	leader.Parse(szBuf, iLen);
	iLength = leader.GetRecLength();

	TDir dir;
	int iOffset = MARC_LEADER_LENGTH;

	MARCField fld;

	while(iOffset < leader.baseAddressOfData - 1)
	{
	dir.Parse(szBuf + iOffset, MARC_DIR_LENGTH);

	fld.Parse(dir.mTag, szBuf + leader.GetBaseAddressOfData() + dir.iStartPos, dir.iLength);
	fields.push_back(fld);
	
	iOffset += MARC_DIR_LENGTH;
	}

	bValid = true;
	MoveFirst();
	return true;
}


template<class TLeader, class TDir>
MARCField & DynamicMARCRecord<TLeader, TDir>::Field()
{
	return *pos;
}


template<class TLeader, class TDir>
MARCTag DynamicMARCRecord<TLeader, TDir>::Tag()
{
	MARCTag mt;
	mt = pos->mTag;
	return mt;
}

template<class TLeader, class TDir>
void DynamicMARCRecord<TLeader, TDir>::MoveFirst()
{
	pos = fields.begin();
}


template<class TLeader, class TDir>
void DynamicMARCRecord<TLeader, TDir>::MoveNext()
{
	++pos;
}

template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::IsEnd()
{
	return (pos != fields.end()) ?  false : true;
}


template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::FindFirst(char * szTag)
{
	string str = szTag;

	for(; pos != fields.end(); ++pos)
	{
	if(pos->mTag.c_str() == str)
		return true;
	}
	return false;
}

template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::FindNext(char * szTag)
{
	string str = szTag;

	for(++pos; pos != fields.end(); ++pos)
	{
	if(pos->mTag.c_str() == str)
		return true;
	}
	return false;
}

/*
template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::IsValid()
{
	return bValid;
}*/

template<class TLeader, class TDir>
string DynamicMARCRecord<TLeader, TDir>::GetTitleString(char * szTag)
{
	string str;

	MoveFirst();

	if(FindFirst(szTag))
	{
	str = Field().GetString();
	}

	MoveFirst();
	return str;
}

/*
template<class TLeader, class TDir>	
int DynamicMARCRecord<TLeader, TDir>::GetRecordLength()
{
	return iLength;
}*/

template<class TLeader, class TDir>
bool DynamicMARCRecord<TLeader, TDir>::operator<(const DynamicMARCRecord<TLeader, TDir> & rec) const
{
	return (sort < rec.sort);
}

template<class TLeader, class TDir>
DynamicMARCRecord<TLeader, TDir> & DynamicMARCRecord<TLeader, TDir>::operator=(const DynamicMARCRecord<TLeader, TDir> & rec)
{
	if(this != &rec)
	{
	leader = rec.leader;
	bValid = rec.bValid;
	sortstring = rec.sortstring;
	iLength = rec.iLength;
	pos = rec.pos;
	fields = rec.fields;
	}

	return *this;
}


#endif
