/****************************************************
	MARC Template Library 

  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: mark_basedow@hotmail.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/


#ifndef _MARCEVENTOBJECT_H
#define _MARCEVENTOBJECT_H


#include <string>
#include "marcutil.h"
#include <map>


using namespace std;

class MARCEventObject
{
public:
	virtual ~MARCEventObject() {}

	virtual bool Parse(unsigned char * szBuf, int iLen) = 0;
	virtual bool IsValid() = 0; 
	virtual int GetRecordLength() = 0;

};



template<class TLeader, class TDir>
class MARCEventRecord : public MARCEventObject
{
public:

	MARCEventRecord();
	virtual ~MARCEventRecord();
	MARCEventRecord(const MARCEventRecord & mr);
		
	bool Parse(unsigned char * szBuf, int iLen);
	
	bool IsValid(); 
	int GetRecordLength();

	virtual void startRecord(const TLeader & leader) = 0; 
    virtual void endRecord() = 0; 
    virtual void controlField(MARCTag & tag, unsigned char * data, int iLen) = 0;
    virtual void dataField(MARCTag & tag, char ind1, char ind2) = 0;
    virtual void subfield(MARCTag & tag, char identifier, unsigned char * data, int iLen) = 0;

//protected:
	
	bool bValid;
	int iLength;

};

template<class TLeader, class TDir>
MARCEventRecord<TLeader, TDir>::MARCEventRecord()
{
	iLength = 0;
	bValid = true;
}


template<class TLeader, class TDir>
MARCEventRecord<TLeader, TDir>::~MARCEventRecord()
{

}

template<class TLeader, class TDir>
MARCEventRecord<TLeader, TDir>::MARCEventRecord(const MARCEventRecord<TLeader, TDir> & rec)
{
	iLength = rec.iLength;
	bValid = rec.bValid;
}

template<class TLeader, class TDir>
bool MARCEventRecord<TLeader, TDir>::Parse(unsigned char * szBuf, int iLen)
{
	TLeader leader;
	leader.Parse(szBuf, iLen);
	iLength = leader.GetRecLength();

	startRecord(leader);

	TDir dir;
	int iOffset = 24;

	while(iOffset < leader.baseAddressOfData - 1)
	{
	dir.Parse(szBuf + iOffset, 12);

	if(dir.mTag.IsData())
		controlField(dir.mTag, szBuf + leader.GetBaseAddressOfData() + dir.iStartPos, dir.iLength) ;
	else
	{
	dataField(dir.mTag, 0, 0);

/*	while()
  
	subfield(MARCTag & tag, char identifier, char * data, int iLen)
*/

	}

	iOffset += 12;
	}

	endRecord();
	return true;
}

template<class TLeader, class TDir>
bool MARCEventRecord<TLeader, TDir>::IsValid()
{
	return bValid;
}

template<class TLeader, class TDir>
int MARCEventRecord<TLeader, TDir>::GetRecordLength()
{
	return iLength;
}





#endif