/****************************************************
	MARC Template Library 

  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: mark_basedow@hotmail.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/

#ifndef _MARCTODC_H
#define _MARCTODC_H

#include <string>
#include "marcrecord.h"
#include <list>
#include "dublincore.h"


using namespace std;



// MARC TO DC CONVERSION STUCTURES //////////////////////////////////////
/*
The conversion to Dublin Core still needs a bit of work.
Need to add support to get information from indicators
and MARC leader.

Possible solution:

  enum MLDC {

struct MDCCONV
{
	char * mDC; 
	char * mQualifier;

	char * mSrcTag;
	char ind1;
	char ind2;
	int lpos; //leader position
	char lvalue; //leader value

	char * mSrcFields;
	bool bRepeatable;
}; 


 */


struct MDCCONV
{
	char * mDC; 
	char * mQualifier;
	char * mSrcTag; 
	char * mSrcFields;
	bool bRepeatable;
}; 

#define BEGIN_MARC_DC_MAP(objName) static MDCCONV objName[] = {
#define MARC_DC_MAP_ENTRY(DC_Element, DC_Qualifier, marcTag, marcSubField, repeatable) { DC_Element, DC_Qualifier, marcTag, marcSubField, repeatable },
#define END_MARC_DC_MAP { 0, 0, 0, 0, false }};


BEGIN_MARC_DC_MAP(MARC21_To_UnqualifiedDC) 
	MARC_DC_MAP_ENTRY("Title", "", "245", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "100", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "110", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "111", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "700", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "600", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "610", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "611", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "630", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "650", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "653", "", true)
	MARC_DC_MAP_ENTRY("Description", "", "500", "", true)
	MARC_DC_MAP_ENTRY("Description", "", "505", "", true)
	MARC_DC_MAP_ENTRY("Publisher", "", "260", "ab", true)
	MARC_DC_MAP_ENTRY("Date", "", "260", "c", true)
	MARC_DC_MAP_ENTRY("Type", "", "655", "", true)
	MARC_DC_MAP_ENTRY("Format", "", "856", "q", true)
	MARC_DC_MAP_ENTRY("Identifier", "", "856", "u", true)
	MARC_DC_MAP_ENTRY("Source", "", "786", "ot", true)
	MARC_DC_MAP_ENTRY("Language", "", "546", "", true)
	MARC_DC_MAP_ENTRY("Relation", "", "530", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "651", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "752", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "651", "", true)
	MARC_DC_MAP_ENTRY("Rights", "", "506", "", true)
	MARC_DC_MAP_ENTRY("Rights", "", "540", "", true)
END_MARC_DC_MAP


BEGIN_MARC_DC_MAP(MARC21_To_QualifiedDC) 
	MARC_DC_MAP_ENTRY("Title", "", "245", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "130", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "210", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "240", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "242", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "246", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "730", "", true)
	MARC_DC_MAP_ENTRY("Title", "Alternative", "740", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "100", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "110", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "111", "", true)
	MARC_DC_MAP_ENTRY("Creator", "", "700", "", true)
	MARC_DC_MAP_ENTRY("Subject", "LCC", "050", "", true)
	MARC_DC_MAP_ENTRY("Subject", "DDC", "082", "", true)
	MARC_DC_MAP_ENTRY("Subject", "UDC", "080", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "600", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "610", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "611", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "630", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "650", "", true)
	MARC_DC_MAP_ENTRY("Subject", "", "653", "", true)
	MARC_DC_MAP_ENTRY("Description", "", "500", "", true)
	MARC_DC_MAP_ENTRY("Description", "TableofContents", "505", "", true)
	MARC_DC_MAP_ENTRY("Publisher", "", "260", "ab", true)
	MARC_DC_MAP_ENTRY("Date", "Created", "260", "cg", true)
	MARC_DC_MAP_ENTRY("Date", "Issued", "260", "c", true)
	MARC_DC_MAP_ENTRY("Type", "", "655", "", true)
	MARC_DC_MAP_ENTRY("Format", "", "856", "q", true)
	MARC_DC_MAP_ENTRY("Identifier", "", "856", "u", true)
	MARC_DC_MAP_ENTRY("Source", "", "786", "ot", true)
	MARC_DC_MAP_ENTRY("Language", "", "546", "", true)
	MARC_DC_MAP_ENTRY("Relation", "", "530", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "651", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "752", "", true)
	MARC_DC_MAP_ENTRY("Coverage", "", "651", "", true)
	MARC_DC_MAP_ENTRY("Rights", "", "506", "", true)
	MARC_DC_MAP_ENTRY("Rights", "", "540", "", true)
END_MARC_DC_MAP



template<class TRecType>
bool MARC_To_DublinCore(DublinCore & dcOut, TRecType & rec, MDCCONV * conv)
{
	int i = 0;

	string str;

	while(conv[i].mDC != 0)
	{
		rec.MoveFirst();
		str = "";

		if(rec.FindFirst(conv[i].mSrcTag))
		{
			do{
			//if no subfields get entire field
			if(!strlen(conv[i].mSrcFields)) 
			{
				str = rec.Field().GetString();
			}
			else // get requested selected subfield
			{
				int t = 0;
				while(conv[i].mSrcFields[t])
				{
				str += rec.Field().GetSubField(conv[i].mSrcFields[t]);
				++t;
				}
			}
			dcOut.Add(conv[i].mDC, conv[i].mQualifier, (char *)str.c_str()); 
			}while(rec.FindNext(conv[i].mSrcTag));
		}
	++i;
	}

	return true;
}




#endif