
#include "static_marcfield.h"


Static_MARCField::Static_MARCField()
{
	buf = 0;
	iLength = 0;

}

Static_MARCField::~Static_MARCField()
{
	if(buf) delete [] buf;

}

Static_MARCField::Static_MARCField(const Static_MARCField & f)
{
	if(f.buf)
	{
	iLength = f.iLength;
	buf = new unsigned char [iLength];
	memcpy(buf, f.buf, iLength);
	buf[iLength - 1] = 0;
	}
	else
	{
	buf = 0;
	iLength = 0;
	}
	mTag = f.mTag;
}

Static_MARCField & Static_MARCField::operator=(const Static_MARCField & f)
{
	if(this != &f)
	{
		if(f.buf)
		{
		iLength = f.iLength;
		buf = new unsigned char [iLength];
		memcpy(buf, f.buf, iLength);
		buf[iLength - 1] = 0;
		}
		else
		{
		buf = 0;
		iLength = 0;
		}
		mTag = f.mTag;
	}	
	return *this;
}

bool Static_MARCField::Parse(const MARCTag & tag, unsigned char * szBuf, int iLen)
{
	if(buf) delete [] buf;
	
	mTag = tag;

	iLength = iLen;
	buf = new unsigned char [iLength];
	memcpy(buf, szBuf, iLength);
	buf[iLength - 1] = 0;

	return true;
}

void Static_MARCField::MoveFirst()
{
	if(mTag.IsControl())
		iOffset = 0;
	else
	{
	iOffset = 0;
	MoveNext();
	}
}

void Static_MARCField::MoveNext()
{
	while(iOffset < iLength)
	{
	if(buf[iOffset] == MARC_ST) return;
	++iOffset;
	}
}

bool Static_MARCField::IsEnd()
{
	return (iOffset < iLength) ? false : true;
}


char Static_MARCField::GetIndicator(int i)
{
	if(!buf) return 0;

	if(i > iLength) return 0; 

	if(mTag.IsControl()) return 0;

	return buf[i];
}


char Static_MARCField::SubFieldCode()
{
	return buf[iOffset + 1];
}


string Static_MARCField::SubFieldString()
{
	string str;
	
	int i = iOffset;
	while(i < iLength)
	{
	if((buf[i] == MARC_ST) || (buf[i] == MARC_FT))
		break;
	++i;
	}

	if(i > iOffset)
	{
	int l = i - iOffset;
	char * p = new char [l + 1];
	memcpy(p, buf + iOffset, l);
	p[l] = 0;
	str = p;
	delete [] p;
	}
	return str;

}

string Static_MARCField::GetString(char * szFormat, bool bIndicator)
{
	string str;

	if(!buf) return str;

	if(mTag.IsControl())
	{
	str = (char *)buf;
	}
	else
	{
		if(szFormat)
		{
			if(bIndicator)
			{
			char it[3];
			if(strlen((const char *)buf) > 2)
			{
			memcpy(it, buf, 2);
			it[2] = 0;
			str = it;
			str += " ";
			}
			}

			int i = 0;
			while(szFormat[i])
			{
			if(szFormat[i] == 32)
				str += " ";
			else
				str += GetSubField(szFormat[i]);
			++i;
			}
		}
		else
		{
		unsigned char * tmpBuf = new unsigned char [iLength + 1];

		if(!bIndicator)
		{
		memcpy(tmpBuf, buf + 4, iLength - 4);
		tmpBuf[iLength - 5] = 0;
		}
		else
		{
		memcpy(tmpBuf, buf, iLength);
		tmpBuf[iLength - 1] = 0;
		}

		RemoveSubFieldDelimiters(tmpBuf, iLength); 

		str = (char *)tmpBuf;
		delete [] tmpBuf;
		}
	}

	return str;
}



string Static_MARCField::GetSubField(char c)
{
	string str;
	unsigned char * start = 0, * end = 0;

	int i = 0;
	while(i < iLength)
	{
		if(start)
		{
		if((buf[i] == 30) || (buf[i] == 31))
		{
		end = buf + i;
		int l = end - start;
		char * p = new char [l + 1];
		memcpy(p, start, l);
		p[l] = 0;
		str = p;
		delete [] p;
		break;
		}

		}

		if((buf[i - 1] == 31) && (buf[i] == c))
		{
		start = buf + i + 1;
		}

	++i;
	}

	return str;
}
