/****************************************************
	MARC Template Library 

  Version 0.3 - Released 18th January 2002
  Version 0.2 - Released 14th January 2002
  Version 0.1 - Released 8th January 2002


	Copyright Mark Basedow 2002.
	Email: mark_basedow@hotmail.com

	Please read licence.txt that is
	distributed with this file.

  ***************************************************/

#ifndef _STATIC_MARCRECORD_H
#define _STATIC_MARCRECORD_H


#include "marcrecord.h"
#include "static_marcfield.h"
#include <string>
#include "marcutil.h"
#include <map>
#include "marcsort.h"


using namespace std;


template<class TLeader, class TDir>  
class StaticMARCRecord : public MARCRecord<TLeader, TDir>
{
public:

	StaticMARCRecord();
	virtual ~StaticMARCRecord();
	StaticMARCRecord(const StaticMARCRecord<TLeader, TDir> & mr);
		
	bool Parse(unsigned char * szBuf, int iLen);
	
	MARCTag Tag();
	MARCField & Field();
	
	string Data();

	void MoveFirst();
	void MoveNext();
	bool IsEnd();
	bool FindFirst(char * szTag);
	bool FindNext(char * szTag);

	string GetTitleString(char * szTag = "245");
	bool SetSortString(char * szTag);

	StaticMARCRecord<TLeader, TDir> & operator=(const StaticMARCRecord<TLeader, TDir> & rec);
	bool operator<(const StaticMARCRecord<TLeader, TDir> & rec) const;


//protected:


	int iLength;
	unsigned char * buf;
	multimap<MARCTag, TDir> directory;
	multimap<MARCTag, TDir>::iterator pos;

	Static_MARCField mf;  
};

template<class TLeader, class TDir>
StaticMARCRecord<TLeader, TDir>::StaticMARCRecord()
{
	buf = 0;
	iLength = 0;
}


template<class TLeader, class TDir>
StaticMARCRecord<TLeader, TDir>::~StaticMARCRecord()
{
	if(buf) delete [] buf;
	buf = 0;
	iLength = 0;
}


template<class TLeader, class TDir>
StaticMARCRecord<TLeader, TDir>::StaticMARCRecord(const StaticMARCRecord<TLeader, TDir> & mr)
{
	iLength = mr.iLength;
	bValid = mr.bValid;
	mf = mr.mf;
	leader = mr.leader;
	directory = mr.directory;
	pos = directory.begin();

	if(mr.buf)
	{
	buf = new unsigned char [iLength + 1];
	memcpy(buf, mr.buf, iLength);
	buf[iLength] = 0;
	}	
}

template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::Parse(unsigned char * szBuf, int iLen)
{
	directory.clear();

	if(buf) delete [] buf;
	buf = 0;
	bValid = true;

	if(!leader.Parse(szBuf, iLen)) return false;
	iLength = leader.GetRecLength();
	if(iLen < iLength) return false;

	if(MARC_RT != (int)szBuf[iLength - 1]) return false;

	buf = new unsigned char [iLength + 1];
	memcpy(buf, szBuf, iLength);
	buf[iLength] = 0;

	TDir dir;
	int iOffset = MARC_LEADER_LENGTH;

	while(iOffset < leader.baseAddressOfData - 1)
	{
	dir.Parse(buf + iOffset, MARC_DIR_LENGTH);

	directory.insert(make_pair(dir.mTag, dir));
	iOffset += MARC_DIR_LENGTH;
	}

	MoveFirst();
	return true;
}

template<class TLeader, class TDir>
MARCField & StaticMARCRecord<TLeader, TDir>::Field()
{
	mf.Parse((MARCTag)pos->first, buf + pos->second.iStartPos + leader.baseAddressOfData, pos->second.iLength);
	return mf;
}

template<class TLeader, class TDir>
MARCTag StaticMARCRecord<TLeader, TDir>::Tag()
{
	MARCTag m;

	if(!IsEnd())
	{
	m = pos->first;
	}
	return m;
}

template<class TLeader, class TDir>
string StaticMARCRecord<TLeader, TDir>::Data()
{
	string str;
	str = (char *)buf;
	return str;
}

template<class TLeader, class TDir>
void StaticMARCRecord<TLeader, TDir>::MoveFirst()
{
	pos = directory.begin();
}


template<class TLeader, class TDir>
void StaticMARCRecord<TLeader, TDir>::MoveNext()
{
	++pos;
}

template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::IsEnd()
{
	return (pos != directory.end()) ?  false : true;
}

template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::FindFirst(char * szTag)
{
	MARCTag mt;
	mt = szTag;

	pos = directory.find(mt);
	
	return !IsEnd();
}

template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::FindNext(char * szTag)
{
	MARCTag mt;
	mt = szTag;

	for(++pos; pos != directory.end(); ++pos)
	{
	if(pos->first == mt)
		return true;
	}
	
	return false;
}



/*
template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::IsValid()
{ 
	return bValid; 
}

template<class TLeader, class TDir>
int StaticMARCRecord<TLeader, TDir>::GetRecordLength() 
{ 
	return iLength; 
}

*/

template<class TLeader, class TDir>
StaticMARCRecord<TLeader, TDir> & StaticMARCRecord<TLeader, TDir>::operator=(const StaticMARCRecord<TLeader, TDir> & rec)
{
	if(this != &rec)
	{
	iLength = rec.iLength;
	bValid = rec.bValid;
	mf = rec.mf;
	leader = rec.leader;
	directory = rec.directory;
	pos = directory.begin();

	if(rec.buf)
	{
	buf = new unsigned char [iLength + 1];
	memcpy(buf, rec.buf, iLength);
	buf[iLength] = 0;
	}	
	}

	return *this;
}

template<class TLeader, class TDir>
string StaticMARCRecord<TLeader, TDir>::GetTitleString(char * szTag)
{
	string str;

	MoveFirst();

	if(FindFirst(szTag))
	{
	str = Field().GetString();
	}

	MoveFirst();
	return str;
}

template<class TLeader, class TDir>
bool StaticMARCRecord<TLeader, TDir>::operator<(const StaticMARCRecord<TLeader, TDir> & rec) const
{
	return false;
}

#endif
